local function list2(params)
	assert(type(params) == "table", "bad argument #1 to 'list2' (table expected, got "..type(params)..")")
	-- the Action callback is only applied to the textbox
	params.expand = params.expand or 'NO'
	params.list = params.list or {}
	params.font = params.font or Font.Default
	params.value = params.value or ''
	params.active = params.active or 'YES'

	local dropdown, textbox, zbox
	
	dropdown = iup.list{dropdown = 'YES', size = params.size, font = params.font, expand = params.expand, fgcolor = '0 0 0 0', visible_items = params.visible_items,
		action = function(self, t, i, v) textbox.value = t; iup.SetFocus(textbox); if (params.action) then textbox:action(nil, t) end end
	}
	for i, v in ipairs(params.list) do dropdown[i] = v; if (v == params.value) then dropdown.value = i end; end
	if (not params.size) then 
		iup.Map(dropdown)
		params.size = tonumber(dropdown.size:match("(%d+)x")) - params.font
	else
		params.size = params.size - params.font
	end
	textbox = iup.text{size = params.size, font = params.font, bgcolor = params.bgcolor, expand = params.expand, value = params.value,
		action = function(self, c, after) if (params.action) then params.action(self, c, after) end end
	}
	zbox = iup.zbox{
		--dropdown must be before textbox within the zbox to create the proper z-order
		dropdown,
		textbox,
		all = 'YES',
		alignment = 'NW',
	}

	function zbox:Activate(b)
		if (b) then
			dropdown.active = "YES"
			textbox.active = "YES"; textbox.bgcolor = params.bgcolor
		else
			dropdown.active = "NO"
			textbox.active = "NO"; textbox.bgcolor = params.bginactive
		end
	end

	function zbox:Clear()
		dropdown.value = 0
		textbox.value = ""
	end
	
	function zbox:SetAction(f)
		assert(type(f) == "function", "bad argument #1 to 'SetAction' (function expected, got "..type(f)..")")
		params.action = f
	end

	if (params.active:upper() == "NO") then zbox:Activate(false) end
	
	return zbox, textbox, dropdown
end

-- 3 buttons, save/discard/cancel type 
local function msgdlgtemplate3(msg, button1text, button1action, button2text, button2action, button3text, button3action, dlgbgcolor, alignment)
	local dlg
	local infolabel = iup.label{font=Font.H1,title=msg or "Title", alignment=alignment}
	local button1 = iup.stationbutton{title=button1text or "Save", action=button1action}
	local button2 = iup.stationbutton{title=button2text or "Discard", action=button2action}
	local button3 = iup.stationbutton{title=button3text or "Cancel", action=button3action}

	dlg = iup.dialog{
		iup.hbox{
			iup.fill{},
			iup.vbox{
				iup.fill{},
				iup.stationhighopacityframe{
					bgcolor=dlgbgcolor,
					iup.stationhighopacityframebg{
						bgcolor=dlgbgcolor,
						iup.vbox{
							infolabel,
							iup.hbox{
								iup.fill{},
								button1,
								iup.fill{},
								button2,
								iup.fill{},
								button3,
								iup.fill{},
							},
						},
						expand="NO",
					},
				},
				iup.fill{},
			},
			iup.fill{},
		},
		--defaultenter = button1,
		defaultesc = button3,
		fullscreen="YES",
		bgcolor = "0 0 0 128 *",
		topmost="YES",
	}

	function dlg:SetMessage(msg, button1text, callback1, button2text, callback2, button3text, callback3, alignment)
		infolabel.alignment = alignment or "ALEFT"
		infolabel.title = msg or ""
		button1.title = button1text or "Save"
		if callback1 then button1.action = callback1 end
		button2.title = button2text or "Discard"
		if callback2 then button2.action = callback2 end
		button3.title = button3text or "Cancel"
		if callback3 then button3.action = callback3 end
		dlg.size = nil
	end

	return dlg
end

return list2, msgdlgtemplate3